from cyclone import web
from oonib.input import handlers
from oonib.config import config

inputAPI = [
    (r"/input", handlers.InputListHandler),
    (r"/input/([a-f0-9]{64})", handlers.InputDescHandler),
    (r"/input/([a-f0-9]{64})/file$", web.StaticFileHandler,
     {"path": config.main.input_dir}),
]



import tempfile
from oonib import log
from oonib.config import config
from twisted.internet import reactor, endpoints
import os

from random import randint
import socket

from txtorcon import TCPHiddenServiceEndpoint, TorConfig
from txtorcon import launch_tor

from txtorcon import __version__ as txtorcon_version
if tuple(map(int, txtorcon_version.split('.'))) < (0, 9, 0):
    """
    Fix for bug in txtorcon versions < 0.9.0 where TCPHiddenServiceEndpoint
    listens on all interfaces by default.
    """
    def create_listener(self, proto):
        self._update_onion(self.hiddenservice.dir)
        self.tcp_endpoint = endpoints.TCP4ServerEndpoint(self.reactor,
                                                         self.listen_port,
                                                         interface='127.0.0.1')
        d = self.tcp_endpoint.listen(self.protocolfactory)
        d.addCallback(self._add_attributes).addErrback(self._retry_local_port)
        return d
    TCPHiddenServiceEndpoint._create_listener =  create_listener

def randomFreePort(addr="127.0.0.1"):
    """
    Args:

        addr (str): the IP address to attempt to bind to.

    Returns an int representing the free port number at the moment of calling

    Note: there is no guarantee that some other application will attempt to
    bind to this port once this function has been called.
    """
    free = False
    while not free:
        port = randint(1024, 65535)
        s = socket.socket()
        try:
            s.bind((addr, port))
            free = True
        except:
            pass
        s.close()
    return port

def txSetupFailed(failure):
    log.err("Setup failed")
    log.exception(failure)

def startTor(torconfig):
    def updates(prog, tag, summary):
        print("%d%%: %s" % (prog, summary))

    if config.main.socks_port:
        torconfig.SocksPort = config.main.socks_port
    if config.main.control_port:
        torconfig.ControlPort = config.main.control_port
    if config.main.tor2webmode:
        torconfig.Tor2webMode = 1
        torconfig.CircuitBuildTimeout = 60
    if config.main.tor_datadir is None:
        temporary_data_dir = tempfile.mkdtemp()
        log.warn("Option 'tor_datadir' in oonib.conf is unspecified!")
        log.warn("Using %s" % temporary_data_dir)
        torconfig.DataDirectory = temporary_data_dir
    else:
        if os.path.exists(config.main.tor_datadir):
            torconfig.DataDirectory = os.path.abspath(config.main.tor_datadir)
        else:
            raise Exception

    tor_log_file = os.path.join(torconfig.DataDirectory, "tor.log")
    torconfig.Log = ["notice stdout", "notice file %s" % tor_log_file]
    torconfig.save()
    if not hasattr(torconfig, 'ControlPort'):
        control_port = int(randomFreePort())
        torconfig.ControlPort = control_port
        config.main.control_port = control_port

    if not hasattr(torconfig, 'SocksPort'):
        socks_port = int(randomFreePort())
        torconfig.SocksPort = socks_port
        config.main.socks_port = socks_port

    torconfig.save()

    if config.main.tor_binary is not None:
        d = launch_tor(torconfig, reactor,
                       tor_binary=config.main.tor_binary,
                       progress_updates=updates)
    else:
        d = launch_tor(torconfig, reactor, progress_updates=updates)
    return d



import json
import random
import string

from twisted.internet import protocol, defer

from cyclone.web import RequestHandler, Application

from twisted.protocols import policies, basic
from twisted.web.http import Request

from oonib import log, randomStr


class SimpleHTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
    """
    This is a simplified version of twisted.web.http.HTTPChannel to overcome
    header lowercase normalization. It does not actually implement the HTTP
    protocol, but only the subset of it that we need for testing.

    What this HTTP channel currently does is process the HTTP Request Line and
    the Request Headers and returns them in a JSON datastructure in the order
    we received them.

    The returned JSON dict looks like so:

    {
        'request_headers':
            [['User-Agent', 'IE6'], ['Content-Length', 200]]
        'request_line':
            'GET / HTTP/1.1'
    }
    """
    requestFactory = Request
    __first_line = 1
    __header = ''
    __content = None

    length = 0
    maxHeaders = 500
    maxHeaderLineLength = 16384
    requestLine = ''

    timeOut = 60 * 60 * 12

    def __init__(self):
        self.headers = []
        self.requests = []

    def connectionMade(self):
        self.setTimeout(self.timeOut)

    def lineReceived(self, line):
        if (len(self.__header) + len(line)) >= self.maxHeaderLineLength \
                and not self.__first_line:
            log.err("Maximum header length reached.")
            return self.transport.loseConnection()

        if self.__first_line:
            self.requestLine = line
            self.__first_line = 0
        elif line == '':
            # We have reached the end of the headers.
            if self.__header:
                self.headerReceived(self.__header)
            self.__header = ''
            self.allHeadersReceived()
            self.setRawMode()
        elif line[0] in ' \t':
            # This is to support header field value folding over multiple lines
            # as specified by rfc2616.
            self.__header += '\n'+line
        else:
            if self.__header:
                self.headerReceived(self.__header)
            self.__header = line

    def headerReceived(self, line):
        try:
            header, data = line.split(':', 1)
            self.headers.append((header, data.strip()))
        except:
            log.err("Got malformed HTTP Header request field")
            log.err("%s" % line)

        if len(self.headers) >= self.maxHeaders:
            log.err("Maximum number of headers received.")
            self.closeConnection()

    def allHeadersReceived(self):
        headers_dict = {}
        for k, v in self.headers:
            if k not in headers_dict:
                headers_dict[k] = []
            headers_dict[k].append(v)

        response = {
            'request_headers': self.headers,
            'request_line': self.requestLine,
            'headers_dict': headers_dict
        }
        json_response = json.dumps(response)
        self.transport.write('HTTP/1.1 200 OK\r\n\r\n')
        self.transport.write('%s' % json_response)

        self.closeConnection()

    def closeConnection(self):
        if self._TimeoutMixin__timeoutCall and \
                not self._TimeoutMixin__timeoutCall.called:
            self._TimeoutMixin__timeoutCall.cancel()
            self._TimeoutMixin__timeoutCall = None
        self.transport.loseConnection()


class HTTPReturnJSONHeadersHelper(protocol.ServerFactory):
    protocol = SimpleHTTPChannel

    def buildProtocol(self, addr):
        return self.protocol()


class HTTPTrapAll(RequestHandler):
    def _execute(self, transforms, *args, **kwargs):
        self._transforms = transforms
        defer.maybeDeferred(self.prepare).addCallbacks(
            self._execute_handler,
            lambda f: self._handle_request_exception(f.value),
            callbackArgs=(args, kwargs))

    def _execute_handler(self, r, args, kwargs):
        if not self._finished:
            args = [self.decode_argument(arg) for arg in args]
            kwargs = dict((k, self.decode_argument(v, name=k))
                          for (k, v) in kwargs.iteritems())

            # This is where we do the patching
            # XXX this is somewhat hackish
            d = defer.maybeDeferred(self.all, *args, **kwargs)
            d.addCallbacks(self._execute_success, self._execute_failure)
            self.notifyFinish().addCallback(self.on_connection_close)


class HTTPRandomPage(HTTPTrapAll):
    """
    This generates a random page of arbitrary length and containing the string
    selected by the user.
    /<length>/<keyword>
    XXX this is currently disabled as it is not of use to any test.
    """
    isLeaf = True

    def _gen_random_string(self, length):
        return ''.join(random.choice(string.letters) for x in range(length))

    def genRandomPage(self, length=100, keyword=None):
        data = self._gen_random_string(length/2)
        if keyword:
            data += keyword
        data += self._gen_random_string(length - length/2)
        data += '\n'
        return data

    def all(self, length, keyword):
        self.set_header('Content-Disposition', 'attachment; filename="%s.txt"' % randomStr(10))
        length = 100
        if length > 100000:
            length = 100000
        self.write(self.genRandomPage(length, keyword))

HTTPRandomPageHelper = Application([
    # XXX add regexps here
    (r"/(.*)/(.*)", HTTPRandomPage)
])



from cyclone import web

from oonib.main.api import mainAPI
from oonib.deck.api import deckAPI
from oonib.report.api import reportAPI
from oonib.input.api import inputAPI
from oonib.policy.api import policyAPI
from oonib.bouncer.api import bouncerAPI

from oonib.config import config

ooniBouncer = None
ooniBackendAPI = []
ooniBackendAPI += reportAPI

if config.main.input_dir:
    ooniBackendAPI += inputAPI

if config.main.deck_dir:
    ooniBackendAPI += deckAPI

if config.main.policy_file:
    ooniBackendAPI += policyAPI

if config.main.bouncer_file:
    ooniBouncer = web.Application(bouncerAPI, debug=True, name='bouncer')

ooniBackendAPI += mainAPI
ooniBackend = web.Application(ooniBackendAPI, debug=True, name='collector')



from oonib.report import handlers

reportAPI = [
    (r"/report/([a-zA-Z0-9_\-]+)/close", handlers.CloseReportHandlerFile),
    (r"/report/([a-zA-Z0-9_\-]+)", handlers.UpdateReportHandlerFile),
    (r"/report", handlers.NewReportHandlerFile),
    (r"/pcap", handlers.PCAPReportHandler),
]



import yaml

from oonib import errors as e
from oonib.config import config


class Policy(object):
    nettest = None
    input = None

    def __init__(self):
        with open(config.main.policy_file) as f:
            p = yaml.safe_load(f)
        self.input = []
        self.nettest = []
        if 'nettest' in p:
            self.nettest = list(p['nettest'])
        if 'input' in p:
            self.input = list(p['input'])

    def validateInputHash(self, input_hash):
        valid = False
        if not self.input:
            valid = True
        for i in self.input:
            if input_hash == i['id']:
                valid = True
                break
        if not valid:
            raise e.InvalidInputHash

    def validateNettest(self, nettest_name):
        # XXX add support for version checking too.
        if self.nettest:
            if not any(nt['name'] == nettest_name for nt in self.nettest):
                raise e.InvalidNettestName



import sys
import os
import socket
import json
import shutil

from twisted.internet import reactor, defer
from twisted.trial import unittest

from oonib.config import config

from cyclone import httpclient


def random_unused_port(bind_address='127.0.0.1'):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('127.0.0.1', 0))
    port = s.getsockname()[1]
    s.close()
    return port

reports = {}


def mock_initialize(self):
    self.report_dir = '.'
    self.archive_dir = '.'
    self.reports = reports
    self.policy_file = None
    self.helpers = {}
    self.stale_time = 10


class HandlerTestCase(unittest.TestCase):
    app = None
    _port = None
    _listener = None
    config_filename = ''

    @property
    def port(self):
        if not self._port:
            self._port = random_unused_port()
        return self._port

    def make_dir(self, dir):
        if not os.path.exists(dir):
            os.makedirs(dir)
            try:
                self.directories.add(dir)
            except AttributeError:
                self.directories = set([dir])

    def setUp(self, *args, **kw):
        self.filenames = set()
        if 'directories' not in dir(self):
            self.directories = set()
        self.old_arguments = sys.argv
        if self.config_filename != '':
            sys.argv = ['test_oonib', '-c', self.config_filename]
            config.load()
        if self.app:
            self._listener = reactor.listenTCP(self.port, self.app)
        return super(HandlerTestCase, self).setUp()

    def tearDown(self):
        sys.argv = self.old_arguments
        for filename in self.filenames:
            if os.path.exists(filename):
                os.remove(filename)
        for dir in self.directories:
            shutil.rmtree(dir)
        if self._listener:
            for report in reports.values():
                try:
                    report.delayed_call.cancel()
                except:
                    pass
            self._listener.stopListening()

    @defer.inlineCallbacks
    def request(self, path, method="GET", postdata=None, headers={}):
        url = "http://localhost:%s%s" % (self.port, path)
        if isinstance(postdata, dict):
            postdata = json.dumps(postdata)

        response = yield httpclient.fetch(url, method=method,
                                          postdata=postdata,
                                          headers=headers)
        defer.returnValue(response)



from twisted.names import client, server, dns

from oonib.config import config


class DNSTestHelper(server.DNSServerFactory):
    def __init__(self, authorities=None,
                 caches=None, clients=None,
                 verbose=0):
        try:
            host, port = config.helpers.dns.split(':')
            port = int(port)
        # XXX remove this when we have configuration file versioning.
        # https://github.com/TheTorProject/ooni-probe/issues/190
        except:
            host, port = '8.8.8.8', 53
        resolver = client.Resolver(servers=[(host, port)])
        server.DNSServerFactory.__init__(self, authorities=authorities,
                                         caches=caches, clients=[resolver],
                                         verbose=verbose)

    def handleQuery(self, message, protocol, address):
        server.DNSServerFactory.handleQuery(self, message, protocol, address)


class DNSResolverDiscovery(server.DNSServerFactory):
    """
    This test helper is used to discover the IP address of the resolver being
    used by a ooniprobe client.
    To use it you should set it up on a machine that has been delegated as the
    authoritative name server for a specific subdomain.
    You can do so by adding the following to your zone file:

        mysubdomain                   IN    NS    ns.mysubdomain.example.org.
        ns.mysubdomain.example.org    IN    A     10.42.42.42

    Replace 10.42.42.42 with the IP address of the machine running oonib.

    You will then be able to perform A lookups on subdomains of
    mysubdomain.example.org and retrieve in the query answer section the IP
    address of the resolver that was used for performing the request.
    """
    def handleQuery(self, message, protocol, address):
        query = message.queries[0]
        if query.type == dns.A:
            ans = dns.RRHeader(bytes(query.name),
                               payload=dns.Record_A(bytes(address[0]), 0))
            message.answers = [ans]
            message.answer = 1
        self.sendReply(protocol, message, address)



from oonib.handlers import OONIBHandler


class OONIBGlobalHandler(OONIBHandler):
    """
        Global handler to catch all orphaned requests.
    """
    def default(self):
        # XXX: Waiting to a stable release of cyclone which implements the default handler
        return self.write_error(404)

    def get(self):
        return self.default()

    def post(self):
        return self.default()

    def options(self):
        return self.default()



from cyclone.web import HTTPError


class OONIBError(HTTPError):
    status_code = 500
    log_message = 'oonib-error'

    def __init__(self):
        pass


class InvalidRequest(OONIBError):
    status_code = 400
    log_message = 'invalid-request'


class NoHelperFound(OONIBError):
    status_code = 404
    log_message = 'no-helper-found'


class InvalidInputHash(OONIBError):
    status_code = 406
    log_message = 'invalid-input-hash'


class InvalidNettestName(OONIBError):
    status_code = 406
    log_message = 'invalid-nettest-name'


class InputHashNotProvided(OONIBError):
    status_code = 406
    log_message = 'input-hash-not-provided'


class InputDescriptorNotFound(OONIBError):
    status_code = 404
    log_message = 'input-descriptor-not-found'


class InvalidRequestField(OONIBError):
    def __init__(self, field_name):
        self.status_code = 400
        self.log_message = "invalid-request-field %s" % field_name


class MissingRequestField(OONIBError):
    def __init__(self, field_name):
        self.status_code = 400
        self.log_message = "missing-request-field %s" % field_name


class MissingReportHeaderKey(OONIBError):
    def __init__(self, key):
        self.status_code = 406
        self.log_message = "missing-report-header-key %s" % key


class MissingDeckKeys(OONIBError):
    status_code = 400
    log_message = "missing-deck-keys"


class MissingDeck(OONIBError):
    status_code = 400
    log_message = "missing-deck"


class NoDecksConfigured(OONIBError):
    status_code = 501
    log_message = "no-decks-configured"


class InvalidReportHeader(OONIBError):
    def __init__(self, key):
        self.status_code = 406
        self.log_message = "invalid-report-header %s" % key


class ReportNotFound(OONIBError):
    status_code = 404
    log_message = "report-not-found"


class CollectorNotFound(OONIBError):
    status_code = 404
    log_message = "collector-not-found"


class NoValidCollector(OONIBError):
    pass


class TestHelpersKeyMissing(OONIBError):
    status_code = 400
    log_message = "test-helpers-key-missing"


class TestHelpersOrNetTestsKeyMissing(OONIBError):
    status_code = 400
    log_message = "test-helpers-or-net-test-key-missing"


class TestHelperNotFound(OONIBError):
    status_code = 404
    log_message = "test-helper-not-found"


class ConfigFileNotSpecified(Exception):
    pass


class ConfigFileDoesNotExist(Exception):
    pass


class InvalidReportDirectory(Exception):
    pass


class InvalidArchiveDirectory(Exception):
    pass


class InvalidInputDirectory(Exception):
    pass


class InvalidDeckDirectory(Exception):
    pass


class InvalidTimestampFormat(Exception):
    pass



"""
In here we shall keep track of all variables and objects that should be
instantiated only once and be common to pieces of GLBackend code.
"""

__version__ = '1.1.3'

__all__ = ['Storage', 'randomStr']

import string
from random import SystemRandom
random = SystemRandom()


class Storage(dict):
    """
    A Storage object is like a dictionary except `obj.foo` can be used
    in addition to `obj['foo']`.

        >>> o = Storage(a=1)
        >>> o.a
        1
        >>> o['a']
        1
        >>> o.a = 2
        >>> o['a']
        2
        >>> del o.a
        >>> o.a
    """
    def __getattr__(self, key):
        return self.get(key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError, k:
            raise AttributeError(k)

    def __repr__(self):
        return '<Storage ' + dict.__repr__(self) + '>'

    def __getstate__(self):
        return dict(self)

    def __setstate__(self, value):
        self.update(value.items())


def randomStr(length, num=True):
    """
    Returns a random a mixed lowercase, uppercase, alfanumerical (if num True)
    string long length
    """
    chars = string.ascii_lowercase + string.ascii_uppercase
    if num:
        chars += string.digits
    return ''.join(random.choice(chars) for x in range(length))



import json

from twisted.internet import defer

from cyclone import web

from oonib.test.handler_helpers import HandlerTestCase
from oonib.main.api import mainAPI


class GlobalHandler(HandlerTestCase):
    app = web.Application(mainAPI, name='mainAPI')

    @defer.inlineCallbacks
    def test_global_handler(self):
        response = yield self.request("/foo")
        res = json.loads(response.body)
        self.assertIn("error", res.keys())
        self.assertIn(404, res.values())



from twisted.internet import ssl
from oonib.config import config

class SSLContext(ssl.DefaultOpenSSLContextFactory):
    def __init__(self, *args, **kw):
        ssl.DefaultOpenSSLContextFactory.__init__(self,
                config.helpers.ssl.private_key,
                config.helpers.ssl.certificate)




from oonib.collector import handlers

reportAPI = [
    (r"/report/([a-zA-Z0-9_\-]+)/close", handlers.CloseReportHandlerFile),
    (r"/report/([a-zA-Z0-9_\-]+)", handlers.UpdateReportHandlerFile),
    (r"/report", handlers.NewReportHandlerFile),
    (r"/pcap", handlers.PCAPReportHandler),
]

policyAPI = [
    (r"/policy/nettest", handlers.NetTestPolicyHandler),
    (r"/policy/input", handlers.InputPolicyHandler),
]

inputAPI = [
    (r"/input", handlers.InputListHandler),
    (r"/input/([a-f0-9]{64})", handlers.InputDescHandler),
    (r"/input/([a-f0-9]{64})/file$", handlers.InputFileHandler)
]



import yaml
from oonib import errors as e
from oonib import Storage
from oonib import __version__

from oonib.options import OONIBOptions
import os


class Config(object):
    main = {}
    helpers = {}
    reports = {}
    backend_version = __version__
    opts = OONIBOptions()

    def load(self):
        self.opts.parseOptions()
        try:
            config_file = self.opts['config']
        except KeyError:
            raise e.ConfigFileNotSpecified

        try:
            with open(config_file) as f:
                configuration = yaml.safe_load(f)
        except IOError:
            raise e.ConfigFileDoesNotExist(config_file)

        self.main = Storage(configuration['main'].items())

        self.helpers = Storage()
        for name, helper in configuration['helpers'].items():
            self.helpers[name] = Storage(helper.items())

        self.check_paths()

    def check_paths(self):
        def check_path(directory, complaint):
            if not (directory and os.path.isdir(directory)):
                raise complaint(directory)
        check_path(self.main.report_dir,  e.InvalidReportDirectory)
        check_path(self.main.archive_dir, e.InvalidArchiveDirectory)
        check_path(self.main.input_dir,   e.InvalidInputDirectory)
        check_path(self.main.deck_dir,    e.InvalidDeckDirectory)

backend_version = __version__

config = Config()



from twisted.internet.protocol import Protocol, Factory, ServerFactory

from oonib.config import config
from oonib import log
from oonib.daphn3 import Daphn3Protocol
from oonib.daphn3 import read_pcap, read_yaml


class TCPEchoProtocol(Protocol):
    def dataReceived(self, data):
        self.transport.write(data)


class TCPEchoHelper(Factory):
    """
    A very simple echo protocol implementation
    """
    protocol = TCPEchoProtocol

if config.helpers.daphn3.yaml_file:
    daphn3Steps = read_pcap(config.helpers.daphn3.yaml_file)

elif config.helpers.daphn3.pcap_file:
    daphn3Steps = read_yaml(config.helpers.daphn3.pcap_file)

else:
    daphn3Steps = [
        {'client': 'client_packet'},
        {'server': 'server_packet'}
    ]


class Daphn3ServerProtocol(Daphn3Protocol):
    def nextStep(self):
        log.debug("Moving on to next step in the state walk")
        self.current_data_received = 0
        # Python why?
        if self.current_step >= (len(self.steps) - 1):
            log.msg("Reached the end of the state machine")
            log.msg("Censorship fingerpint bisected!")
            step_idx, mutation_idx = self.factory.mutation
            log.msg("step_idx: %s | mutation_id: %s" % (step_idx, mutation_idx))
            #self.transport.loseConnection()
            if self.report:
                self.report['mutation_idx'] = mutation_idx
                self.report['step_idx'] = step_idx
            return
        else:
            self.current_step += 1
        if self._current_step_role() == self.role:
            # We need to send more data because we are again responsible for
            # doing so.
            self.sendPayload()

class Daphn3Server(ServerFactory):
    """
    This is the main class that deals with the daphn3 server side component.
    We keep track of global state of every client here.
    Every client is identified by their IP address and the state of mutation is
    stored by using their IP address as a key. This may lead to some bugs if
    two different clients are sharing the same IP, but hopefully the
    probability of such thing is not that likely.
    """
    protocol = Daphn3ServerProtocol
    # step_idx, mutation_idx
    mutation = [0, 0]
    def buildProtocol(self, addr):
        p = self.protocol()
        p.steps = daphn3Steps
        p.role = "server"
        p.factory = self
        return p



import glob
import os
import re
import yaml

from oonib import errors as e
from oonib.handlers import OONIBHandler
from oonib import log
from oonib.config import config


class DeckDescHandler(OONIBHandler):
    def get(self, deckID):
        # note:
        # we don't have to sanitize deckID, because it's already checked
        # against matching a certain pattern in the handler.
        bn = os.path.basename(deckID + '.desc')
        try:
            f = open(os.path.join(config.main.deck_dir, bn))
        except IOError:
            log.err("Deck %s missing" % deckID)
            raise e.MissingDeck
        with f:
            deckDesc = yaml.safe_load(f)

        response = {}
        for k in ['name', 'description', 'version', 'author', 'date']:
            try:
                response[k] = deckDesc[k]
            except KeyError:
                log.err("Deck %s missing required keys!" % deckID)
                raise e.MissingDeckKeys
        self.write(response)


class DeckListHandler(OONIBHandler):
    def get(self):
        if not config.main.deck_dir:
            self.set_status(501)
            raise e.NoDecksConfigured

        path = os.path.abspath(config.main.deck_dir) + "/*"
        decknames = map(os.path.basename, glob.iglob(path))
        decknames = filter(lambda y: re.match("[a-z0-9]{64}.desc", y),
                           decknames)
        deckList = []
        for deckname in decknames:
            with open(os.path.join(config.main.deck_dir, deckname)) as f:
                d = yaml.safe_load(f)
                deckList.append({
                    'id': deckname,
                    'name': d['name'],
                    'description': d['description']
                })
        self.write(deckList)



import yaml

from twisted.internet import protocol, defer

from oonib import log

def read_pcap(filename):
    """
    @param filename: Filesystem path to the pcap.

    Returns:
      [{"client": "\x17\x52\x15"}, {"server": "\x17\x15\x13"}]
    """
    from scapy.all import IP, Raw, rdpcap

    packets = rdpcap(filename)

    """
    pcap assumptions:

    pcap only contains packets exchanged between a Tor client and a Tor
    server.  (This assumption makes sure that there are only two IP addresses
    in the pcap file)

    The first packet of the pcap is sent from the client to the server. (This
    assumption is used to get the IP address of the client.)

    All captured packets are TLS packets: that is TCP session
    establishment/teardown packets should be filtered out (no SYN/SYN+ACK)
    """

    """
    Minimally validate the pcap and also find out what's the client
    and server IP addresses.
    """
    ssl_packets = []
    client_ip_addr = None
    server_ip_addr = None
    for i, packet in enumerate(packets):
        if i == 0:
            client_ip_addr = packet[IP].src
        elif packet[IP].src != client_ip_addr:
            server_ip_addr = packet[IP].src

        try:
            if packet[Raw]:
                ssl_packets.append(packet)
        except IndexError:
            pass

    """Form our list."""
    messages = []
    for packet in ssl_packets:
        if packet[IP].src == client_ip_addr:
            messages.append({"client": str(packet[Raw])})
        elif packet[IP].src == server_ip_addr:
            messages.append({"server": str(packet[Raw])})
        else:
            raise("Detected third IP address! pcap is corrupted.")

    return messages

def read_yaml(filename):
    f = open(filename)
    obj = yaml.safe_load(f)
    f.close()
    return obj

class NoInputSpecified(Exception):
    pass

class StepError(Exception):
    pass

def daphn3MutateString(string, i):
    """
    Takes a string and mutates the ith bytes of it.
    """
    mutated = ""
    for y in range(len(string)):
        if y == i:
            mutated += chr(ord(string[i]) + 1)
        else:
            mutated += string[y]
    return mutated

def daphn3Mutate(steps, step_idx, mutation_idx):
    """
    Take a set of steps and a step index and mutates the step of that
    index at the mutation_idx'th byte.
    """
    mutated_steps = []
    for idx, step in enumerate(steps):
        if idx == step_idx:
            step_string = step.values()[0]
            step_key = step.keys()[0]
            mutated_string = daphn3MutateString(step_string,
                    mutation_idx)
            mutated_steps.append({step_key: mutated_string})
        else:
            mutated_steps.append(step)
    return mutated_steps

class Daphn3Protocol(protocol.Protocol):
    steps = None
    role = "client"
    report = None
    # We use this index to keep track of where we are in the state machine
    current_step = 0
    current_data_received = 0

    # We use this to keep track of the mutated steps
    mutated_steps = None
    d = defer.Deferred()

    def _current_step_role(self):
        return self.steps[self.current_step].keys()[0]

    def _current_step_data(self):
        step_idx, mutation_idx = self.factory.mutation
        log.debug("Mutating %s %s" % (step_idx, mutation_idx))
        mutated_step = daphn3Mutate(self.steps,
                step_idx, mutation_idx)
        log.debug("Mutated packet into %s" % mutated_step)
        return mutated_step[self.current_step].values()[0]

    def sendPayload(self):
        self.debug("Sending payload")
        current_step_role = self._current_step_role()
        current_step_data = self._current_step_data()
        if current_step_role == self.role:
            print "In a state to do shit %s" % current_step_data
            self.transport.write(current_step_data)
            self.nextStep()
        else:
            print "Not in a state to do anything"

    def connectionMade(self):
        print "Got connection"

    def debug(self, msg):
        log.debug("Current step %s" % self.current_step)
        log.debug("Current data received %s" % self.current_data_received)
        log.debug("Current role %s" % self.role)
        log.debug("Current steps %s" % self.steps)
        log.debug("Current step data %s" % self._current_step_data())

    def nextStep(self):
        """
        XXX this method is overwritten individually by client and server transport.
        There is probably a smarter way to do this and refactor the common
        code into one place, but for the moment like this is good.
        """
        pass

    def dataReceived(self, data):
        current_step_role = self.steps[self.current_step].keys()[0]
        log.debug("Current step role %s" % current_step_role)
        if current_step_role == self.role:
            log.debug("Got a state error!")
            raise StepError("I should not have gotten data, while I did, \
                    perhaps there is something wrong with the state machine?")

        self.current_data_received += len(data)
        expected_data_in_this_state = len(self.steps[self.current_step].values()[0])

        log.debug("Current data received %s" %  self.current_data_received)
        if self.current_data_received >= expected_data_in_this_state:
            self.nextStep()

    def nextMutation(self):
        log.debug("Moving onto next mutation")
        # [step_idx, mutation_idx]
        c_step_idx, c_mutation_idx = self.factory.mutation
        log.debug("[%s]: c_step_idx: %s | c_mutation_idx: %s" % (self.role,
            c_step_idx, c_mutation_idx))

        if c_step_idx >= (len(self.steps) - 1):
            log.err("No censorship fingerprint bisected.")
            log.err("Givinig up.")
            self.transport.loseConnection()
            return

        # This means we have mutated all bytes in the step
        # we should proceed to mutating the next step.
        log.debug("steps: %s | %s" % (self.steps, self.steps[c_step_idx]))
        if c_mutation_idx >= (len(self.steps[c_step_idx].values()[0]) - 1):
            log.debug("Finished mutating step")
            # increase step
            self.factory.mutation[0] += 1
            # reset mutation idx
            self.factory.mutation[1] = 0
        else:
            log.debug("Mutating next byte in step")
            # increase mutation index
            self.factory.mutation[1] += 1

    def connectionLost(self, reason):
        self.debug("--- Lost the connection ---")
        self.nextMutation()




from twisted.python import usage


class OONIBOptions(usage.Options):
    synopsis = """%s [options]"""

    longdesc = ("oonib provides the backend component of ooni-probe."
                "oonib provides test helper services and a reporting "
                "backend service. oonib is intended to be run as a "
                "daemon and most options are set in its configuration file")

    optParameters = [["config", "c", "oonib.conf", "Path to config file"]]



import json
from twisted.internet import defer

from oonib.test.handler_helpers import HandlerTestCase
from oonib.testhelpers import http_helpers


class TestHTTPReturnJSONHeaders(HandlerTestCase):
    app = http_helpers.HTTPReturnJSONHeadersHelper()

    @defer.inlineCallbacks
    def test_get_request(self):
        response = yield self.request('/', "GET",
                                      headers={'X-Antani': ['Spam']})
        response_body = json.loads(response.body)
        self.assertIn("headers_dict", response_body)
        self.assertIn("X-Antani", response_body['headers_dict'])
        self.assertIn(["X-Antani", "Spam"], response_body['request_headers'])
        self.assertEqual("GET / HTTP/1.1", response_body['request_line'])


class TestHTTPRandomPage(HandlerTestCase):
    app = http_helpers.HTTPRandomPageHelper

    @defer.inlineCallbacks
    def test_get_request(self):
        response = yield self.request('/100/ham', "GET")
        self.assertEqual(len(response.body), 104)



import shutil
import time
import yaml
import json
import os
import re

from twisted.internet import fdesc, reactor

from oonib import errors as e
from oonib.handlers import OONIBHandler
from oonib.policy.handlers import Policy

from datetime import datetime
from oonib import randomStr, otime, log
from oonib.config import config


def report_file_name(archive_dir, report_details):
    timestamp = datetime.fromtimestamp(report_details['start_time'])
    keys = dict(
        report_details.items(),
        iso8601_timestamp=otime.timestamp(timestamp),
        year=timestamp.strftime("%Y"),
        month=timestamp.strftime("%m"),
        day=timestamp.strftime("%d"),
        hour=timestamp.strftime("%H"),
        minute=timestamp.strftime("%M"),
        second=timestamp.strftime("%S")
    )
    report_file_template = "{probe_cc}/{test_name}-{iso8601_timestamp}-{probe_asn}-probe.yamloo"
    if config.main.report_file_template:
        report_file_template = config.main.report_file_template
    dst_filename = os.path.join(archive_dir, report_file_template.format(**keys))
    dst_dir = os.path.dirname(dst_filename)
    if not os.path.exists(dst_dir):
        os.makedirs(dst_dir)
    return dst_filename


class Report(object):
    delayed_call = None

    def __init__(self, report_id,
                 stale_time,
                 report_dir,
                 archive_dir,
                 reports):
        self.report_id = report_id

        self.stale_time = stale_time
        self.report_dir = report_dir
        self.archive_dir = archive_dir
        self.reports = reports

        self.refresh()

    def refresh(self):
        self.last_updated = time.time()
        if self.delayed_call:
            self.delayed_call.cancel()
        self.delayed_call = reactor.callLater(self.stale_time,
                                              self.stale_check)

    def stale_check(self):
        if (time.time() - self.last_updated) > self.stale_time:
            try:
                self.close()
            except e.ReportNotFound:
                pass

    def close(self):
        def get_report_path(report_id):
            return os.path.join(self.report_dir, report_id)

        report_filename = get_report_path(self.report_id)
        try:
            with open(report_filename) as fd:
                g = yaml.safe_load_all(fd)
                report_details = g.next()
        except IOError:
            raise e.ReportNotFound

        dst_filename = report_file_name(self.archive_dir, report_details)
        shutil.move(report_filename, dst_filename)

        if not self.delayed_call.called:
            self.delayed_call.cancel()
        del self.reports[self.report_id]


def parseUpdateReportRequest(request):
    #db_report_id_regexp = re.compile("[a-zA-Z0-9]+$")

    # this is the regexp for the reports that include the timestamp
    report_id_regexp = re.compile("[a-zA-Z0-9_\-]+$")

    # XXX here we are actually parsing a json object that could be quite big.
    # If we want this to scale properly we only want to look at the test_id
    # field.
    # We are also keeping in memory multiple copies of the same object. A lot
    # of optimization can be done.
    parsed_request = json.loads(request)
    try:
        report_id = parsed_request['report_id']
    except KeyError:
        raise e.MissingField('report_id')

    if not re.match(report_id_regexp, report_id):
        raise e.InvalidRequestField('report_id')

    return parsed_request


def parseNewReportRequest(request):
    """
    Here we parse a new report request.
    """
    version_string = re.compile("[0-9A-Za-z_\-\.]+$")
    name = re.compile("[a-zA-Z0-9_\- ]+$")
    probe_asn = re.compile("AS[0-9]+$")
    test_helper = re.compile("[A-Za-z0-9_\-]+$")

    expected_request = {
        'software_name': name,
        'software_version': version_string,
        'test_name': name,
        'test_version': version_string,
        'probe_asn': probe_asn
    }

    parsed_request = json.loads(request)
    if 'probe_asn' not in parsed_request or not parsed_request['probe_asn']:
        parsed_request['probe_asn'] = 'AS0'

    for k, regexp in expected_request.items():
        try:
            value_to_check = parsed_request[k]
        except KeyError:
            raise e.MissingField(k)

        print "Matching %s with %s | %s" % (regexp, value_to_check, k)
        if re.match(regexp, str(value_to_check)):
            continue
        else:
            raise e.InvalidRequestField(k)

    try:
        requested_test_helper = parsed_request['test_helper']
        if not re.match(test_helper, str(requested_test_helper)):
            raise e.InvalidRequestField('test_helper')
    except KeyError:
        pass

    return parsed_request


def validate_report_header(report_header):
    required_keys = ['probe_asn', 'probe_cc', 'probe_ip', 'software_name',
                     'software_version', 'test_name', 'test_version']
    for key in required_keys:
        if key not in report_header:
            raise e.MissingReportHeaderKey(key)

    if report_header['probe_asn'] is None:
        report_header['probe_asn'] = 'AS0'

    if not re.match('AS[0-9]+$', report_header['probe_asn']):
        raise e.InvalidReportHeader('probe_asn')

    # If no country is known, set it to be ZZ (user assigned value in ISO 3166)
    if report_header['probe_cc'] is None:
        report_header['probe_cc'] = 'ZZ'

    if not re.match('[a-zA-Z]{2}$', report_header['probe_cc']):
        raise e.InvalidReportHeader('probe_cc')

    if not re.match('[a-z_\-]+$', report_header['test_name']):
        raise e.InvalidReportHeader('test_name')

    if not re.match('([0-9]+\.)+[0-9]+$', report_header['test_version']):
        raise e.InvalidReportHeader('test_version')

    return report_header


class ReportHandler(OONIBHandler):

    def initialize(self):
        self.archive_dir = config.main.archive_dir
        self.report_dir = config.main.report_dir
        self.reports = config.reports
        self.policy_file = config.main.policy_file
        self.helpers = config.helpers
        self.stale_time = config.main.stale_time


class UpdateReportMixin(object):

    def updateReport(self, report_id, parsed_request):

        log.debug("Got this request %s" % parsed_request)
        report_filename = os.path.join(self.report_dir,
                                       report_id)

        self.reports[report_id].refresh()

        try:
            with open(report_filename, 'a+') as fd:
                fd.write(parsed_request['content'])
        except IOError:
            e.OONIBError(404, "Report not found")
        self.write({'status': 'success'})


class NewReportHandlerFile(ReportHandler, UpdateReportMixin):

    """
    Responsible for creating and updating reports by writing to flat file.
    """
    inputHashes = None

    def checkPolicy(self):
        policy = Policy()
        for input_hash in self.inputHashes:
            policy.validateInputHash(input_hash)
        policy.validateNettest(self.testName)

    def post(self):
        """
        Creates a new report with the input

        * Request

          {'software_name': 'XXX',
           'software_version': 'XXX',
           'test_name': 'XXX',
           'test_version': 'XXX',
           'probe_asn': 'XXX'
           'content': 'XXX'
           }

          Optional:
            'test_helper': 'XXX'
            'client_ip': 'XXX'

          (not implemented, nor in client, nor in backend)
          The idea behind these two fields is that it would be interesting to
          also collect how the request was observed from the collectors point
          of view.

          We use as a unique key the client_ip address and a time window. We
          then need to tell the test_helper that is selected the client_ip
          address and tell it to expect a connection from a probe in that time
          window.

          Once the test_helper sees a connection from that client_ip it will
          store for the testing session the data that it receives.
          When the probe completes the report (or the time window is over) the
          final report will include also the data collected from the
          collectors view point.

        * Response

          {'backend_version': 'XXX', 'report_id': 'XXX'}

        """
        # Note: the request is being validated inside of parseNewReportRequest.
        report_data = parseNewReportRequest(self.request.body)

        log.debug("Parsed this data %s" % report_data)

        software_name = str(report_data['software_name'])
        software_version = str(report_data['software_version'])

        probe_asn = str(report_data['probe_asn'])
        probe_cc = str(report_data.get('probe_cc', 'ZZ'))

        self.testName = str(report_data['test_name'])
        self.testVersion = str(report_data['test_version'])

        if self.policy_file:
            try:
                self.inputHashes = report_data['input_hashes']
            except KeyError:
                raise e.InputHashNotProvided
            self.checkPolicy()

        if 'content' in report_data:
            content = yaml.safe_load(report_data['content'])
            report_header = validate_report_header(content)

        else:
            content = {
                'software_name': software_name,
                'software_version': software_version,
                'probe_asn': probe_asn,
                'probe_cc': probe_cc,
                'test_name': self.testName,
                'test_version': self.testVersion,
                'input_hashes': self.inputHashes,
                'start_time': time.time()
            }

        content['backend_version'] = config.backend_version

        report_header = yaml.dump(content)
        content = "---\n" + report_header + '...\n'

        if not probe_asn:
            probe_asn = "AS0"

        report_id = otime.timestamp() + '_' \
            + probe_asn + '_' \
            + randomStr(50)

        # The report filename contains the timestamp of the report plus a
        # random nonce
        report_filename = os.path.join(self.report_dir, report_id)

        response = {
            'backend_version': config.backend_version,
            'report_id': report_id
        }

        requested_helper = report_data.get('test_helper')

        if requested_helper:
            try:
                response['test_helper_address'] = self.helpers[
                    requested_helper].address
            except KeyError:
                raise e.TestHelperNotFound

        self.reports[report_id] = Report(report_id,
                                         self.stale_time,
                                         self.report_dir,
                                         self.archive_dir,
                                         self.reports)

        self.writeToReport(report_filename, content)

        self.write(response)

    def writeToReport(self, report_filename, data):
        with open(report_filename, 'w+') as fd:
            fdesc.setNonBlocking(fd.fileno())
            fdesc.writeToFD(fd.fileno(), data)

    def put(self):
        """
        Update an already existing report.

          {
           'report_id': 'XXX',
           'content': 'XXX'
          }
        """
        parsed_request = parseUpdateReportRequest(self.request.body)
        report_id = parsed_request['report_id']

        self.updateReport(report_id, parsed_request)


class UpdateReportHandlerFile(ReportHandler, UpdateReportMixin):

    def post(self, report_id):
        try:
            parsed_request = json.loads(self.request.body)
        except ValueError:
            raise e.InvalidRequest
        self.updateReport(report_id, parsed_request)


class CloseReportHandlerFile(ReportHandler):

    def get(self):
        pass

    def post(self, report_id):
        if report_id in self.reports:
            self.reports[report_id].close()
        else:
            raise e.ReportNotFound


class PCAPReportHandler(ReportHandler):

    def get(self):
        pass

    def post(self):
        pass



import os
import json
import yaml

from twisted.internet import defer

from cyclone import web

from oonib.report.handlers import report_file_name
from oonib.report.api import reportAPI
from oonib.test.handler_helpers import HandlerTestCase, mock_initialize

sample_report_entry = """---
agent: agent
input: null
requests:
- request:
    body: null
    headers:
    - - ACCePT-LAnGuagE
    - ['en-US,en;q=0.8']
    - - aCCEPT-ENcODInG
    - ['gzip,deflate,sdch']
    - - aCcEPT
    - ['text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8']
    - - User-AGeNt
    - ['Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.7) Gecko/20091221
        Firefox/3.5.7']
    - - aCCEpt-cHArSEt
    - ['ISO-8859-1,utf-8;q=0.7,*;q=0.3']
    - - HOsT
    - [KIXnnZDJfGKRNab.com]
    method: GET
    url: http://12.34.56.78
response:
    body: '{"headers_dict": {"ACCePT-LAnGuagE": ["en-US,en;q=0.8"], "aCCEPT-ENcODInG":
    ["gzip,deflate,sdch"], "HOsT": ["KIXnnZDJfGKRNab.com"], "aCcEPT": ["text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"],
    "User-AGeNt": ["Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.7)
    Gecko/20091221 Firefox/3.5.7"], "aCCEpt-cHArSEt": ["ISO-8859-1,utf-8;q=0.7,*;q=0.3"],
    "Connection": ["close"]}, "request_line": "GET / HTTP/1.1", "request_headers":
    [["Connection", "close"], ["ACCePT-LAnGuagE", "en-US,en;q=0.8"], ["aCCEPT-ENcODInG",
    "gzip,deflate,sdch"], ["aCcEPT", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"],
    ["User-AGeNt", "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.7)
    Gecko/20091221 Firefox/3.5.7"], ["aCCEpt-cHArSEt", "ISO-8859-1,utf-8;q=0.7,*;q=0.3"],
    ["HOsT", "KIXnnZDJfGKRNab.com"]]}'
    code: 200
    headers: []
socksproxy: null
tampering:
header_field_name: false
header_field_number: false
header_field_value: false
header_name_capitalization: false
header_name_diff: []
request_line_capitalization: false
total: false
...
"""

sample_report_header = """---
input_hashes: []
options: []
probe_asn: AS0
probe_cc: ZZ
probe_city: null
probe_ip: 127.0.0.1
software_name: ooniprobe
software_version: 1.1.0
start_time: 0
test_name: fake_test
test_version: 0.1.0
...
"""

for _, handler in reportAPI:
    handler.initialize = mock_initialize


class TestReport(HandlerTestCase):
    app = web.Application(reportAPI, name='reportAPI')

    @defer.inlineCallbacks
    def update_report(self, report_id, content=sample_report_entry):
        data = {
            'content': content
        }
        response = yield self.request(
            '/report/%s' % report_id,
            "POST", data)
        defer.returnValue(response)


    @defer.inlineCallbacks
    def test_create_valid_report(self):
        data = {
            'software_name': 'ooni-test',
            'software_version': '0.1',
            'test_name': 'some-test',
            'test_version': '0.1',
            'probe_asn': 'AS0'
        }
        response = yield self.request('/report', "POST", data)
        response_body = json.loads(response.body)
        self.assertIn('backend_version', response_body)
        self.assertIn('report_id', response_body)
        self.filenames.add(response_body['report_id'])

    @defer.inlineCallbacks
    def test_create_valid_report_with_content(self):
        data = {
            'software_name': 'ooni-test',
            'software_version': '0.1',
            'test_name': 'some-test',
            'test_version': '0.1',
            'probe_asn': 'AS0',
            'content': sample_report_header
        }
        response = yield self.request('/report', "POST", data)
        response_body = json.loads(response.body)
        self.assertIn('backend_version', response_body)
        self.assertIn('report_id', response_body)
        self.filenames.add(response_body['report_id'])

    @defer.inlineCallbacks
    def test_create_invalid_report(self):
        data = {
            'software_name': 'ooni-test',
            'software_version': '0.1',
            'test_name': 'some-test',
            'test_version': '0.1',
            'probe_asn': 'XXX'
        }
        response = yield self.request('/report', "POST", data)
        response_body = json.loads(response.body)
        self.assertIn('error', response_body)
        self.assertEqual(response_body['error'],
                         'invalid-request-field probe_asn')

    @defer.inlineCallbacks
    def test_create_and_update_report(self):
        report_header = {
            'software_name': 'ooni-test',
            'software_version': '0.1',
            'test_name': 'some-test',
            'test_version': '0.1',
            'probe_asn': 'AS0'
        }
        response = yield self.request('/report', "POST", report_header)
        response_body = json.loads(response.body)
        self.assertIn('backend_version', response_body)
        self.assertIn('report_id', response_body)

        report_id = response_body['report_id']
        response = yield self.update_report(report_id)
        response_body = json.loads(response.body)

        with open(report_id) as f:
            self.filenames.add(report_id)
            written_report = yaml.safe_load_all(f)

            written_report_header = written_report.next()
            for key in report_header.keys():
                self.assertEqual(written_report_header[key], report_header[key])
            self.assertEqual(yaml.safe_load(sample_report_entry),
                             written_report.next())

    @defer.inlineCallbacks
    def test_create_update_and_close_report(self):
        report_header = {
            'software_name': 'ooni-test',
            'software_version': '0.1',
            'test_name': 'some-test',
            'test_version': '0.1',
            'probe_asn': 'AS0'
        }
        response = yield self.request('/report', "POST", report_header)
        response_body = json.loads(response.body)
        self.assertIn('backend_version', response_body)
        self.assertIn('report_id', response_body)

        report_entry_count = 100

        report_id = response_body['report_id']
        for i in range(report_entry_count):
            yield self.update_report(report_id)

        with open(report_id) as f:
            written_report = yaml.safe_load_all(f)

            written_report_header = written_report.next()
            for key in report_header.keys():
                self.assertEqual(written_report_header[key],
                                 report_header[key])

            self.assertEqual(yaml.safe_load(sample_report_entry),
                             written_report.next())

        response = yield self.request('/report/%s/close' % report_id, "POST")

        written_report_path = report_file_name(".", written_report_header)
        with open(written_report_path) as f:
            self.filenames.add(written_report_path)
            written_report = yaml.safe_load_all(f)
            written_report.next()

            for i in range(report_entry_count):
                self.assertEqual(yaml.safe_load(sample_report_entry),
                                 written_report.next())

        self.directories.add('ZZ')



# ooni backend
# ************
#
# This is the backend system responsible for running certain services that are
# useful for censorship detection.
#
# In here we start all the test helpers that are required by ooniprobe and
# start the report collector

from distutils.version import LooseVersion

from oonib.api import ooniBackend, ooniBouncer
from oonib.config import config
from oonib.onion import startTor
from oonib.testhelpers import dns_helpers, ssl_helpers
from oonib.testhelpers import http_helpers, tcp_helpers

import os

from twisted.application import internet, service
from twisted.internet import reactor
from twisted.names import dns

from txtorcon import TCPHiddenServiceEndpoint, TorConfig
from txtorcon import __version__ as txtorcon_version

if config.main.uid and config.main.gid:
    application = service.Application('oonibackend', uid=config.main.uid,
                                      gid=config.main.gid)
else:
    application = service.Application('oonibackend')

multiService = service.MultiService()

if config.helpers['ssl'].port:
    print "Starting SSL helper on %s" % config.helpers['ssl'].port
    ssl_helper = internet.SSLServer(int(config.helpers['ssl'].port),
                                    http_helpers.HTTPReturnJSONHeadersHelper(),
                                    ssl_helpers.SSLContext(config))
    multiService.addService(ssl_helper)

# Start the DNS Server related services
if config.helpers['dns'].tcp_port:
    print "Starting TCP DNS Helper on %s" % config.helpers['dns'].tcp_port
    tcp_dns_helper = internet.TCPServer(int(config.helpers['dns'].tcp_port),
                                        dns_helpers.DNSTestHelper())
    multiService.addService(tcp_dns_helper)

if config.helpers['dns'].udp_port:
    print "Starting UDP DNS Helper on %s" % config.helpers['dns'].udp_port
    udp_dns_factory = dns.DNSDatagramProtocol(dns_helpers.DNSTestHelper())
    udp_dns_helper = internet.UDPServer(int(config.helpers['dns'].udp_port),
                                        udp_dns_factory)
    multiService.addService(udp_dns_helper)

if config.helpers['dns_discovery'].udp_port:
    print ("Starting UDP DNS Discovery Helper on %s" %
           config.helpers['dns_discovery'].udp_port)
    udp_dns_discovery = internet.UDPServer(int(config.helpers['dns_discovery'].udp_port),
                                           dns.DNSDatagramProtocol(
                                               dns_helpers.DNSResolverDiscovery()
                                           ))
    multiService.addService(udp_dns_discovery)

if config.helpers['dns_discovery'].tcp_port:
    print ("Starting TCP DNS Discovery Helper on %s" %
           config.helpers['dns_discovery'].tcp_port)
    tcp_dns_discovery = internet.TCPServer(int(config.helpers['dns_discovery'].tcp_port),
                                           dns_helpers.DNSResolverDiscovery())
    multiService.addService(tcp_dns_discovery)


# XXX this needs to be ported
# Start the OONI daphn3 backend
if config.helpers['daphn3'].port:
    print "Starting Daphn3 helper on %s" % config.helpers['daphn3'].port
    daphn3_helper = internet.TCPServer(int(config.helpers['daphn3'].port),
                                       tcp_helpers.Daphn3Server())
    multiService.addService(daphn3_helper)


if config.helpers['tcp-echo'].port:
    print "Starting TCP echo helper on %s" % config.helpers['tcp-echo'].port
    tcp_echo_helper = internet.TCPServer(int(config.helpers['tcp-echo'].port),
                                         tcp_helpers.TCPEchoHelper())
    multiService.addService(tcp_echo_helper)

if config.helpers['http-return-json-headers'].port:
    print "Starting HTTP return request helper on %s" % config.helpers['http-return-json-headers'].port
    http_return_request_helper = internet.TCPServer(
        int(config.helpers['http-return-json-headers'].port),
        http_helpers.HTTPReturnJSONHeadersHelper())
    multiService.addService(http_return_request_helper)


# add the tor collector service here
if config.main.tor_hidden_service:
    torconfig = TorConfig()
    d = startTor(torconfig)

    def getHSEndpoint(data_dir):
        if LooseVersion(txtorcon_version) >= LooseVersion('0.10.0'):
            return TCPHiddenServiceEndpoint(reactor,
                torconfig, 80, hidden_service_dir=data_dir)
        else:
            return TCPHiddenServiceEndpoint(reactor,
                torconfig, 80, data_dir=data_dir)

    def printOnionEndpoint(endpointService):
        print("Exposed %s Tor hidden service on httpo://%s" %
                (endpointService.name, endpointService.endpoint.onion_uri))

    def addCollector(torControlProtocol):
        data_dir = os.path.join(torconfig.DataDirectory, 'collector')
        collector_service = internet.StreamServerEndpointService(
                getHSEndpoint(data_dir),
                ooniBackend)
        collector_service.setName('collector')
        multiService.addService(collector_service)
        collector_service.startService()
        return collector_service

    d.addCallback(addCollector)
    d.addCallback(printOnionEndpoint)

    if ooniBouncer:
        def addBouncer(torControlProtocol):
            data_dir = os.path.join(torconfig.DataDirectory, 'bouncer')
            bouncer_service = internet.StreamServerEndpointService(
                    getHSEndpoint(data_dir),
                    ooniBouncer)
            bouncer_service.setName('bouncer')
            multiService.addService(bouncer_service)
            bouncer_service.startService()
            return bouncer_service

        d.addCallback(addBouncer)
        d.addCallback(printOnionEndpoint)
else:
    if ooniBouncer:
        bouncer_service = internet.TCPServer(8888, ooniBouncer, interface="127.0.0.1")
        multiService.addService(bouncer_service)
        bouncer_service.startService()
    collector_service = internet.TCPServer(8889, ooniBackend, interface="127.0.0.1")
    multiService.addService(collector_service)
    collector_service.startService()



import types

from cyclone import escape
from cyclone import web

from oonib import log


class OONIBHandler(web.RequestHandler):

    def write_error(self, status_code, exception=None, **kw):
        self.set_status(status_code)
        if hasattr(exception, 'log_message') and exception.log_message is not None:
            self.write({'error': exception.log_message})
        elif 400 <= status_code < 600:
            self.write({'error': status_code})
        else:
            log.exception(exception)
            self.write({'error': 'error'})

    def write(self, chunk):
        """
        This is a monkey patch to RequestHandler to allow us to serialize also
        json list objects.
        """
        if isinstance(chunk, types.ListType):
            chunk = escape.json_encode(chunk)
            web.RequestHandler.write(self, chunk)
            self.set_header("Content-Type", "application/json")
        else:
            web.RequestHandler.write(self, chunk)



import json
import yaml
import random
from oonib import errors as e
from oonib.handlers import OONIBHandler
from oonib.config import config


class Bouncer(object):

    def __init__(self, bouncer_file):
        self.bouncerFilename = bouncer_file
        self.load()

    def load(self):
        with open(self.bouncerFilename) as f:
            self.bouncerFile = yaml.safe_load(f)
        self.updateKnownHelpers()
        self.updateKnownCollectors()

    def updateKnownCollectors(self):
        """
        Initialize the list of all known collectors
        """
        self.knownCollectorsWithPolicy = []
        self.knownCollectorsWithoutPolicy = []
        for collectorName, content in self.bouncerFile['collector'].items():
            if content.get('policy') is not None and \
                    collectorName not in self.knownCollectorsWithPolicy:
                self.knownCollectorsWithPolicy.append(collectorName)
            elif content.get('policy') is None and \
                    collectorName not in self.knownCollectorsWithoutPolicy:
                self.knownCollectorsWithoutPolicy.append(collectorName)

    def updateKnownHelpers(self):
        self.knownHelpers = {}
        for collectorName, helpers in self.bouncerFile['collector'].items():
            for helperName, helperAddress in helpers['test-helper'].items():
                if helperName not in self.knownHelpers.keys():
                    self.knownHelpers[helperName] = []

                self.knownHelpers[helperName].append({
                    'collector-name': collectorName,
                    'helper-address': helperAddress
                })

    def getHelperAddresses(self, helper_name):
        """
        Returns a dict keyed on the collector address of known test helpers.
        example:
         {
            'httpo://thirteenchars1.onion': '127.0.0.1',
            'httpo://thirteenchars2.onion': '127.0.0.2',
            'httpo://thirteenchars3.onion': '127.0.0.3'
         }
        """
        try:
            helpers = self.knownHelpers[helper_name]
        except KeyError:
            raise e.TestHelperNotFound

        helpers_dict = {}
        for helper in helpers:
            helpers_dict[helper['collector-name']] = helper['helper-address']

        return helpers_dict

    def filterHelperAddresses(self, requested_helpers):
        """
        Returns a dict of collectors that support all the requested_helpers.

        Example:
        requested_helpers = ['a', 'b', 'c']
        will return:
         {
            'a': {
                'address': '127.0.0.1',
                'collector': 'httpo://thirteenchars1.onion'
            },
            'b': {
                'address': '127.0.0.1:8081',
                'collector': 'httpo://thirteenchars1.onion'
            },
            'c': {
                'address': 'http://127.0.0.1',
                'collector': 'httpo://thirteenchars2.onion'
            },
            'default': {
                'collector': 'httpo://thirteenchars1.onion'
            }
         }

         or

         {'error': 'test-helper-not-found'}

         if no valid helper was found

        """
        response = {}
        for helper_name in requested_helpers:
            try:
                # If we can, try to pick the same collector.
                choices = self.getHelperAddresses(helper_name)
                for item in response.values():
                    c = item['collector']
                    if c in choices:
                        break
                # Or default to a random selection
                else:
                    c = random.choice(choices.keys())

                response[helper_name] = {'collector': c, 'address': choices[c]}

            except e.TestHelperNotFound:
                response = {'error': 'test-helper-not-found'}
                return response

        if len(self.knownCollectorsWithoutPolicy) > 0:
            default_collector = random.choice(
                self.knownCollectorsWithoutPolicy)
        else:
            default_collector = None
        response['default'] = {'collector': default_collector}
        return response

    def collectorAccepting(self, net_test_name, input_hashes, test_helpers):
        for collector_address in self.knownCollectorsWithPolicy:
            collector = self.bouncerFile['collector'][collector_address]
            supported_net_tests = [x['name'] for x in
                                   collector['policy']['nettest']]
            supported_input_hashes = [x['id'] for x in
                                      collector['policy']['input']]
            if net_test_name not in supported_net_tests:
                continue
            if any([input_hash not in supported_input_hashes for input_hash in
                    input_hashes]):
                continue
            if all([x in collector['test-helper'].keys() for x in
                    test_helpers]):
                return collector_address
        if len(self.knownCollectorsWithoutPolicy) > 0:
            return random.choice(self.knownCollectorsWithoutPolicy)
        else:
            raise e.CollectorNotFound

    def filterByNetTests(self, requested_nettests):
        """
        Here we will return a list containing test helpers and collectors for
        the required nettests.
        We give favour to the collectors that have a stricter policy and if
        those fail we will resort to using the collectors with a more lax
        policy.
        """
        nettests = []
        for requested_nettest in requested_nettests:
            collector = self.collectorAccepting(
                requested_nettest['name'],
                requested_nettest['input-hashes'],
                requested_nettest['test-helpers'])
            test_helpers = {}
            for test_helper in requested_nettest['test-helpers']:
                test_helpers[test_helper] = self.bouncerFile['collector'][collector]['test-helper'][test_helper]

            nettest = {
                'name': requested_nettest['name'],
                'version': requested_nettest['version'],
                'input-hashes': requested_nettest['input-hashes'],
                'test-helpers': test_helpers,
                'collector': collector,
            }
            nettests.append(nettest)
        return {'net-tests': nettests}


class BouncerHandlerBase(OONIBHandler):
    def initialize(self):
        self.bouncer = Bouncer(config.main.bouncer_file)

    def load_query(self):
        if not 'query' in dir(self):
            try:
                self.query = json.loads(self.request.body)
            except ValueError:
                raise e.InvalidRequest


class BouncerTestHelpers(BouncerHandlerBase):
    def post(self):
        self.load_query()
        if 'test-helpers' in self.query:
            requested_helpers = self.query['test-helpers']
            if not isinstance(requested_helpers, list):
                raise e.InvalidRequest
            response = self.bouncer.filterHelperAddresses(requested_helpers)
        else:
            raise e.InvalidRequest

        self.write(response)


class BouncerNetTests(BouncerHandlerBase):
    def post(self):
        self.load_query()
        if 'net-tests' in self.query:
            response = self.bouncer.filterByNetTests(self.query['net-tests'])
        else:
            raise e.InvalidRequest

        self.write(response)


class BouncerQueryHandler(BouncerNetTests, BouncerTestHelpers):
    def post(self):
        self.load_query()
        if 'test-helpers' in self.query:
            BouncerTestHelpers.post(self)
        elif 'net-tests' in self.query:
            BouncerNetTests.post(self)
        else:
            raise e.TestHelpersOrNetTestsKeyMissing



from __future__ import with_statement

import os
import sys
from oonib import __version__
from setuptools import setup, find_packages
from setuptools.command import install


def get_requirements():
    with open('requirements.txt', 'r') as f:
        requirements = f.read().splitlines()

    # For urls such as https://hg.secdev.org/scapy/archive/tip.zip#egg=scapy in
    # requirements.txt we need to add the package name to install_requires and
    # the entire url to dependency_links. That way setuptools will be able to
    # satisfy the dependency using that url (as long as it is in standard sdist
    # format, a single .py file or an egg).
    pypi_packages = []
    dependency_links = []
    for package_desc in requirements:
        if package_desc.startswith("#") or package_desc.startswith("-i"):
            continue
        if '#egg=' in package_desc:
            dependency_links.append(package_desc)
            pypi_packages.append(package_desc.split('#egg=')[-1])
        else:
            pypi_packages.append(package_desc)

    return pypi_packages, dependency_links

usr_share_path = '/usr/share'
var_path = '/var'
etc_path = '/etc'
# If this is true then it means we are in a virtualenv
# therefore we should not place our data files inside /usr/share/ooni, but
# place them inside the virtual env system prefix.
if hasattr(sys, 'real_prefix'):
        usr_share_path = os.path.abspath(os.path.join(sys.prefix, 'share'))
        var_path = os.path.abspath(os.path.join(sys.prefix, 'var'))
        etc_path = os.path.abspath(os.path.join(sys.prefix, 'etc'))

data_dir = os.path.join(usr_share_path, 'ooni', 'backend')
spool_dir = os.path.join(var_path, 'spool', 'ooni', 'backend')
var_dir = os.path.join(var_path, 'spool', 'ooni', 'backend')

def install_config_file(self):
    print "Writing the default configuration file to %s" % etc_path
    if not os.path.isdir(etc_path):
        os.makedirs(etc_path)
    import yaml
    example_config = yaml.safe_load(open('oonib.conf.example'))
    example_config['main']['report_dir'] = os.path.join(spool_dir, 'reports')
    example_config['main']['archive_dir'] = os.path.join(spool_dir, 'archive')
    example_config['main']['deck_dir'] = os.path.join(data_dir, 'decks')
    example_config['main']['input_dir'] = os.path.join(data_dir, 'inputs')
    example_config['main']['policy_file'] = os.path.join(var_dir, 'policy.yaml')
    example_config['main']['bouncer_file'] = os.path.join(var_dir, 'bouncer.yaml')
    example_config['main']['tor_datadir'] = os.path.join(var_dir, 'tor')

    with open(os.path.join(etc_path, 'oonibackend.conf'), 'w+') as fw:
        yaml.dump(example_config, fw, default_flow_style=False)

install.install.sub_commands.append(('install_config_file', install_config_file))

data_files = [
    (data_dir, ['oonib.conf.example']),
    (os.path.join(data_dir, 'decks'), ['data/decks/README']),
    (os.path.join(data_dir, 'inputs'), ['data/inputs/Makefile']),
    (os.path.join(spool_dir, 'reports'), ['data/reports/empty.txt']),
    (os.path.join(spool_dir, 'archive'), ['data/archive/empty.txt']),
    (os.path.join(var_dir, 'tor'), ['data/tor/empty.txt']),
    (var_dir, ['data/bouncer.yaml', 'data/policy.yaml'])
]

with open('README.rst') as f:
    readme = f.read()

with open('ChangeLog.rst') as f:
    changelog = f.read()

install_requires, dependency_links = get_requirements()
setup(
    name="oonibackend",
    version=__version__,
    author="The Tor Project, Inc",
    author_email="ooni-dev@torproject.org",
    url="https://ooni.torproject.org",
    license="BSD 2 clause",
    description="Open Observatory of Network Interference Backend",
    long_description=readme + '\n\n' + changelog,
    scripts=["bin/oonib", "bin/oonibadmin", "bin/archive_oonib_reports"],
    packages=find_packages(),
    data_files=data_files,
    install_requires=install_requires,
    dependency_links=dependency_links,
    classifiers=(
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Framework :: Twisted",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Telecommunications Industry",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: BSD :: BSD/OS",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Operating System :: POSIX :: BSD :: NetBSD",
        "Operating System :: POSIX :: BSD :: OpenBSD",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Unix",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Security",
        "Topic :: Security :: Cryptography",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Testing :: Traffic Generation",
        "Topic :: System :: Networking :: Monitoring",
    )
)



from oonib.bouncer import handlers
# XXX: if bouncer is configured
bouncerAPI = [
    # return a collector and helper of the requested type
    (r"/bouncer", handlers.BouncerQueryHandler),
    (r"/bouncer/test-helpers", handlers.BouncerTestHelpers),
    (r"/bouncer/net-tests", handlers.BouncerNetTests),
]



from cyclone import web
from oonib.deck import handlers
from oonib.config import config

deckAPI = [
    (r"/deck", handlers.DeckListHandler),
    (r"/deck/([a-z0-9]{64})$", handlers.DeckDescHandler),
    (r"/deck/([a-z0-9]{64})/file$", web.StaticFileHandler,
     {"path": config.main.deck_dir}),
]



# -*- encoding: utf-8 -*-
#
# :authors: Arturo Filastò, Isis Lovecruft
# :licence: see LICENSE for details
"""
Twisted logger for the ooni backend system.
"""

import os
import sys
import codecs
import logging
import traceback

from twisted.python import log as txlog
from twisted.python import util
from twisted.python.failure import Failure
from twisted.python.logfile import DailyLogFile

from oonib import otime
from oonib.config import config

# Get rid of the annoying "No route found for
# IPv6 destination warnings":
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)


def log_encode(logmsg):
    """
    I encode logmsg (a str or unicode) as printable ASCII. Each case
    gets a distinct prefix, so that people differentiate a unicode
    from a utf-8-encoded-byte-string or binary gunk that would
    otherwise result in the same final output.
    """
    if isinstance(logmsg, unicode):
        return codecs.encode(logmsg, 'unicode_escape')
    elif isinstance(logmsg, str):
        try:
            unicodelogmsg = logmsg.decode('utf-8')
        except UnicodeDecodeError:
            return codecs.encode(logmsg, 'string_escape')
        else:
            return codecs.encode(unicodelogmsg, 'unicode_escape')
    else:
        raise Exception("I accept only a unicode object or a string, "
                        "not a %s object like %r" % (type(logmsg),
                                                     repr(logmsg)))


class LogWithNoPrefix(txlog.FileLogObserver):
    def emit(self, eventDict):
        text = txlog.textFromEventDict(eventDict)
        if text is None:
            return

        util.untilConcludes(self.write, "%s\n" % text)
        util.untilConcludes(self.flush)  # Hoorj!


def start(application_name="oonib"):
    daily_logfile = None

    if not config.main.logfile:
        logfile = 'oonib.log'
    else:
        logfile = config.main.logfile

    log_folder = os.path.dirname(logfile)
    log_filename = os.path.basename(logfile)

    daily_logfile = DailyLogFile(log_filename, log_folder)

    txlog.msg("Starting %s on %s (%s UTC)" % (application_name,
                                              otime.prettyDateNow(),
                                              otime.utcPrettyDateNow()))
    txlog.msg("oonib version %s" % config.backend_version)
    txlog.startLoggingWithObserver(LogWithNoPrefix(sys.stdout).emit)
    txlog.addObserver(txlog.FileLogObserver(daily_logfile).emit)


def stop():
    print "Stopping OONI"


def msg(msg, *arg, **kw):
    print "%s" % log_encode(msg)


def debug(msg, *arg, **kw):
    if config.main.get('debug'):
        print "[D] %s" % log_encode(msg)


def warn(msg, *arg, **kw):
    print "[W] %s" % log_encode(msg)


def err(msg, *arg, **kw):
    print "[!] %s" % log_encode(msg)


def exception(error):
    """
    Error can either be an error message to print to stdout and to the logfile
    or it can be a twisted.python.failure.Failure instance.
    """
    if isinstance(error, Failure):
        error.printTraceback()
    else:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback)


class LoggerFactory(object):
    """
    This is a logger factory to be used by oonib
    """
    def __init__(self, options):
        pass

    def start(self, application):
        start("OONIB")

    def stop(self):
        txlog.msg("Stopping OONIB")



import time
from oonib import errors as e
from datetime import datetime


def utcDateNow():
    """
    Returns the datetime object of the current UTC time.
    """
    return datetime.utcnow()


def utcTimeNow():
    """
    Returns seconds since epoch in UTC time, it's of type float.
    """
    return time.mktime(time.gmtime())


def dateToTime(date):
    """
    Takes as input a datetime object and outputs the seconds since epoch.
    """
    return time.mktime(date.timetuple())


def prettyDateNow():
    """
    Returns a good looking string for the local time.
    """
    return datetime.now().ctime()


def utcPrettyDateNow():
    """
    Returns a good looking string for utc time.
    """
    return datetime.utcnow().ctime()


def timeToPrettyDate(time_val):
    return time.ctime(time_val)


def fromTimestamp(s):
    """
    Converts a string that is output from the timestamp function back to a
    datetime object

    Args:
        s (str): a ISO8601 formatted string.
            ex. 1912-06-23T101234Z"

    Note: we currently only support parsing strings that are generated from the
        timestamp function and have no intention in supporting the full
        standard.
    """
    try:
        date_part, time_part = s.split('T')
        hours, minutes, seconds = time_part[:2], time_part[2:4], time_part[4:6]
        year, month, day = date_part.split('-')
    except:
        raise e.InvalidTimestampFormat(s)

    return datetime(int(year),
                    int(month),
                    int(day),
                    int(hours),
                    int(minutes),
                    int(seconds))


def timestamp(t=None):
    """
    The timestamp for ooni reports follows ISO 8601 in
    UTC time format.
    We do not inlcude ':' and include seconds.

    Example:

        if the current date is "10:12:34 AM, June 23 1912" (datetime(1912, 6,
            23, 10, 12, 34))

        the timestamp will be:

           "1912-06-23T101234Z"

    Args:
        t (datetime): a datetime object representing the
            time to be represented (*MUST* be expressed
            in UTC).

        If not specified will default to the current time
        in UTC.
    """
    if t is None:
        t = datetime.utcnow()
    ISO8601 = "%Y-%m-%dT%H%M%SZ"
    return t.strftime(ISO8601)



# -*- encoding: utf8 -*-
import json

from twisted.internet import defer

from cyclone import web

from oonib.bouncer.api import bouncerAPI
from oonib.test.handler_helpers import HandlerTestCase

fake_bouncer_file = """
collector:
  fake_address:
    policy:
      input:
      - {id: fake_id}
      nettest:
      - {name: fake_nettest, version: 1.0}
      - {name: another_fake_nettest, version: 2.0}
    test-helper: {fake_test_helper: 'fake_hostname'}
"""

fake_default_collector = """
collector:
  fake_address:
    policy:
      input:
      - {id: fake_id}
      nettest:
      - {name: fake_nettest, version: 1.0}
      - {name: another_fake_nettest, version: 2.0}
    test-helper: {fake_test_helper: 'fake_hostname'}
  default_collector:
    test-helper: {fake_test_helper: 'fake_hostname'}
"""

fake_bouncer_file_multiple_collectors = """
collector:
  fake_addressA:
    policy:
      input:
      - {id: fake_id}
      nettest:
      - {name: fake_nettest, version: 1.0}
      - {name: another_fake_nettest, version: 2.0}
    test-helper: {fake_test_helper: 'fake_hostname'}

  fake_addressB:
    policy:
      input:
      - {id: fake_id}
      nettest:
      - {name: fake_nettest, version: 1.0}
      - {name: another_fake_nettest, version: 2.0}
    test-helper: {fake_test_helper: 'fake_hostname'}
"""

fake_for_test_helper_request = """
collector:
  fake_addressA:
    policy:
      input:
      - {id: fake_id}
      nettest:
      - {name: fake_nettest, version: 1.0}
    test-helper: {fake_test_helper: 'fake_hostname', exotic_test_helper: 'fake_hostname'}

  fake_addressB:
    test-helper: {fake_test_helper: 'fake_hostname'}
"""

reports_dir = 'data/reports'
archive_dir = 'data/archive'
input_dir = 'data/inputs'
decks_dir = 'data/decks'
bouncer_filename = 'fake_bouncer_file.yaml'
fake_config_file = """
main:
    report_dir: %s
    archive_dir: %s
    input_dir: %s
    deck_dir: %s
    bouncer_file: %s

helpers:
    fake_test_helper:
         attribute: value
""" % (reports_dir, archive_dir, input_dir, decks_dir, bouncer_filename)


class BaseTestBouncer(HandlerTestCase):
    def setUp(self, *args, **kw):
        self.make_dir(reports_dir)
        self.make_dir(archive_dir)
        self.make_dir(input_dir)
        self.make_dir(decks_dir)

        self.config_filename = 'fake_config.conf'
        with open(self.config_filename, 'w') as config_file:
            config_file.write(fake_config_file)

        super(BaseTestBouncer, self).setUp()
        self.filenames.add(bouncer_filename)
        self.filenames.add(self.config_filename)


class TestBouncer(BaseTestBouncer):
    app = web.Application(bouncerAPI, name='bouncerAPI')

    def setUp(self, *args, **kw):
        with open(bouncer_filename, 'w') as bouncer_file:
            bouncer_file.write(fake_bouncer_file)
        super(TestBouncer, self).setUp()

    @defer.inlineCallbacks
    def test_void_net_tests(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": ['fake_test_helper'],
                    "input-hashes": [],
                    "name": '',
                    "version": '',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('collector-not-found', response_body['error'])

    @defer.inlineCallbacks
    def test_net_tests(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('version', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['version'], '1.0')
        self.assertIn('input-hashes', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['input-hashes']), 0)
        self.assertIn('test-helpers', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['test-helpers']), 0)
        self.assertIn('collector', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['collector'], 'fake_address')

    @defer.inlineCallbacks
    def test_backward_compatibility(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('version', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['version'], '1.0')
        self.assertIn('input-hashes', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['input-hashes']), 0)
        self.assertIn('test-helpers', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['test-helpers']), 0)
        self.assertIn('collector', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['collector'], 'fake_address')

    @defer.inlineCallbacks
    def test_multiple_net_tests(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'another_fake_nettest',
                    "version": '1.0',
                }
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 2)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('name', response_body['net-tests'][1])
        self.assertEqual(response_body['net-tests'][1]['name'], 'another_fake_nettest')

    @defer.inlineCallbacks
    def test_invalid_net_test(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'invalid_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('collector-not-found', response_body['error'])

    @defer.inlineCallbacks
    def test_net_tests_with_input(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": ['fake_id'],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('input-hashes', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['input-hashes']), 1)
        self.assertEqual(response_body['net-tests'][0]['input-hashes'][0], 'fake_id')

    @defer.inlineCallbacks
    def test_net_tests_with_input_invalid_id(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": ['invalid_id'],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('collector-not-found', response_body['error'])

    @defer.inlineCallbacks
    def test_net_tests_with_test_helper(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": ['fake_test_helper'],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('test-helpers', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['test-helpers']), 1)
        self.assertEqual(response_body['net-tests'][0]['test-helpers']['fake_test_helper'], 'fake_hostname')

    @defer.inlineCallbacks
    def test_net_tests_with_invalid_test_helper(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": ['invalid_test_helper'],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('collector-not-found', response_body['error'])

    @defer.inlineCallbacks
    def test_invalid_request(self):
        data = {
            'something_weird': 'nsa'
        }

        response = yield self.request('/bouncer', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('test-helpers-or-net-test-key-missing', response_body['error'])


class TestDefaultCollector(BaseTestBouncer):
    app = web.Application(bouncerAPI, name='bouncerAPI')

    def setUp(self, *args, **kw):
        with open(bouncer_filename, 'w') as bouncer_file:
            bouncer_file.write(fake_default_collector)
        super(TestDefaultCollector, self).setUp()

    @defer.inlineCallbacks
    def test_default_collector(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'imaginary_nettest',
                    "version": '1.0',
                }
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'imaginary_nettest')
        self.assertIn('version', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['version'], '1.0')
        self.assertIn('input-hashes', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['input-hashes']), 0)
        self.assertIn('test-helpers', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['test-helpers']), 0)
        self.assertIn('collector', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['collector'], 'default_collector')


class TestMultipleCollectors(BaseTestBouncer):
    app = web.Application(bouncerAPI, name='bouncerAPI')

    def setUp(self, *args, **kw):
        with open(bouncer_filename, 'w') as bouncer_file:
            bouncer_file.write(fake_bouncer_file_multiple_collectors)
        super(TestMultipleCollectors, self).setUp()

    @defer.inlineCallbacks
    def test_multiple_collectors(self):
        data = {
            'net-tests': [
                {
                    "test-helpers": [],
                    "input-hashes": [],
                    "name": 'fake_nettest',
                    "version": '1.0',
                },
            ]
        }

        response = yield self.request('/bouncer/net-tests', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('net-tests', response_body)
        self.assertEqual(len(response_body['net-tests']), 1)
        self.assertIn('name', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['name'], 'fake_nettest')
        self.assertIn('version', response_body['net-tests'][0])
        self.assertEqual(response_body['net-tests'][0]['version'], '1.0')
        self.assertIn('input-hashes', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['input-hashes']), 0)
        self.assertIn('test-helpers', response_body['net-tests'][0])
        self.assertEqual(len(response_body['net-tests'][0]['test-helpers']), 0)
        self.assertIn('collector', response_body['net-tests'][0])
        self.assertIn(response_body['net-tests'][0]['collector'], ['fake_addressA', 'fake_addressB'])


class TestHelperTests(BaseTestBouncer):
    app = web.Application(bouncerAPI, name='bouncerAPI')

    def setUp(self, *args, **kw):
        with open(bouncer_filename, 'w') as bouncer_file:
            bouncer_file.write(fake_for_test_helper_request)
        super(TestHelperTests, self).setUp()

    @defer.inlineCallbacks
    def test_invalid_helper(self):
        data = {
            'test-helpers': ['invalid_test_helper']
        }
        response = yield self.request('/bouncer/test-helpers', 'POST', data)
        response_body = json.loads(response.body)

        self.assertIn('error', response_body)
        self.assertEqual('test-helper-not-found', response_body['error'])

    @defer.inlineCallbacks
    def test_multiple_collectors(self):
        data = {
            'test-helpers': ['fake_test_helper']
        }

        response = yield self.request('/bouncer/test-helpers', 'POST', data)
        response_body = json.loads(response.body)

        self.assertEqual(len(response_body), 2)
        self.assertIn('fake_test_helper', response_body)
        self.assertIn('collector', response_body['fake_test_helper'])
        self.assertIn(response_body['fake_test_helper']['collector'], ['fake_addressA', 'fake_addressB'])
        self.assertIn('address', response_body['fake_test_helper'])
        self.assertEqual('fake_hostname', response_body['fake_test_helper']['address'])

        self.assertIn('default', response_body)
        self.assertIn('collector', response_body['default'])
        self.assertEqual('fake_addressB', response_body['default']['collector'])

    @defer.inlineCallbacks
    def test_backward_compatibility(self):
        data = {
            'test-helpers': ['fake_test_helper']
        }

        response = yield self.request('/bouncer', 'POST', data)
        response_body = json.loads(response.body)

        self.assertEqual(len(response_body), 2)
        self.assertIn('fake_test_helper', response_body)
        self.assertIn('collector', response_body['fake_test_helper'])
        self.assertIn(response_body['fake_test_helper']['collector'], ['fake_addressA', 'fake_addressB'])
        self.assertIn('address', response_body['fake_test_helper'])
        self.assertEqual('fake_hostname', response_body['fake_test_helper']['address'])

        self.assertIn('default', response_body)
        self.assertIn('collector', response_body['default'])
        self.assertEqual('fake_addressB', response_body['default']['collector'])

    @defer.inlineCallbacks
    def test_multiple_helpers(self):
        data = {
            'test-helpers': ['fake_test_helper', 'exotic_test_helper']
        }

        response = yield self.request('/bouncer/test-helpers', 'POST', data)
        response_body = json.loads(response.body)

        self.assertEqual(len(response_body), 3)
        self.assertIn('fake_test_helper', response_body)
        self.assertIn('exotic_test_helper', response_body)
        self.assertIn('default', response_body)
        self.assertIn(response_body['fake_test_helper']['collector'], ['fake_addressA', 'fake_addressB'])
        self.assertEqual(response_body['exotic_test_helper']['collector'], 'fake_addressA')
        self.assertEqual('fake_addressB', response_body['default']['collector'])



import glob
import os
import yaml

from oonib.handlers import OONIBHandler
from oonib import log
from oonib import errors as e
from oonib.config import config


class InputDescHandler(OONIBHandler):
    def get(self, inputID):
        bn = os.path.basename(inputID) + ".desc"
        try:
            f = open(os.path.join(config.main.input_dir, bn))
        except IOError:
            log.err("No Input Descriptor found for id %s" % inputID)
            raise e.InputDescriptorNotFound
        with f:
            inputDesc = yaml.safe_load(f)

        response = {'id': inputID}
        for k in ['name', 'description', 'version', 'author', 'date']:
            try:
                response[k] = inputDesc[k]
            except KeyError:
                log.err("Invalid Input Descriptor found for id %s" % inputID)
                raise e.InputDescriptorNotFound

        self.write(response)


class InputListHandler(OONIBHandler):
    def get(self):
        path = os.path.abspath(config.main.input_dir) + "/*.desc"
        inputnames = map(os.path.basename, glob.iglob(path))
        inputList = []
        for inputname in inputnames:
            with open(os.path.join(config.main.input_dir, inputname)) as f:
                d = yaml.safe_load(f)
                inputList.append({
                    'id': inputname,
                    'name': d['name'],
                    'description': d['description']
                })
        self.write(inputList)
